package com.loafle.commons.rpc.protocol.json;

import java.io.IOException;
import java.lang.reflect.Type;

import com.google.gson.internal.Primitives;
import com.loafle.commons.rpc.RPCException;
import com.loafle.commons.rpc.protocol.RPCClientNotificationCodec;

import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.JavaType;

/**
 * JSONRPCClientNotificationCodec
 */
public class JSONRPCClientNotificationCodec implements RPCClientNotificationCodec {
  private ObjectMapper objectMapper;
  private JSONClientNotification notification;

  public JSONRPCClientNotificationCodec(ObjectMapper objectMapper, byte[] buff) throws RPCException {
    this.objectMapper = objectMapper;
    try {
      this.notification = this.objectMapper.readValue(buff, JSONClientNotification.class);
    } catch (IOException e) {
      throw new RPCException("cannot unmarsharling", e);
    }
  }

	@Override
	public String method() {
		return this.notification.method;
	}

	@Override
	public Object[] params(Type[] paramTypes) throws RPCException {
    if (null == paramTypes && null == this.notification.params) {
      return null;
    }

    if ((null != paramTypes && null == this.notification.params) || null == paramTypes && null != this.notification.params) {
      throw new RPCException("params is not valied");
    }

    if (paramTypes.length != this.notification.params.size()) {
      throw new RPCException(String.format("The size of params[%d] and types[%d] is not same",
          this.notification.params.size(), paramTypes.length));
    }

    Object[] result = new Object[paramTypes.length];

    for (int i = 0; i < paramTypes.length; i++) {
      Type paramType = paramTypes[i];
      String param = this.notification.params.get(i);

      JavaType targetType = objectMapper.getTypeFactory().constructType(paramType);
      if (!Primitives.isPrimitive(paramType) && !paramType.getTypeName().equals(String.class.getName())) {
        try {
          result[i] = objectMapper.readValue(param, targetType);
        } catch (IOException e) {
          throw new IllegalArgumentException();
        }
      }
      result[i] = objectMapper.convertValue(param, targetType);
    }

    return result;
  }
}