package com.loafle.commons.rpc.protocol.json;

import java.io.IOException;

import com.google.gson.internal.Primitives;
import com.loafle.commons.rpc.RPCException;
import com.loafle.commons.rpc.protocol.RPCClientNotificationCodec;
import com.loafle.commons.rpc.protocol.RPCClientResponseCodec;
import com.loafle.commons.rpc.protocol.RPCError;

import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.JavaType;

/**
 * JSONRPCClientResponseCodec
 */
public class JSONRPCClientResponseCodec implements RPCClientResponseCodec {
  private ObjectMapper objectMapper;
  private JSONClientResponse response;

  public JSONRPCClientResponseCodec(ObjectMapper objectMapper, byte[] buff) throws RPCException {
    this.objectMapper = objectMapper;
    try {
      this.response = this.objectMapper.readValue(buff, JSONClientResponse.class);
    } catch (IOException e) {
      throw new RPCException("cannot unmarsharling", e);
    }
  }

  @Override
  public boolean isNotification() {
    return null == this.response.id && null == this.response.error && null != this.response.result;
  }
  
  @Override
  public RPCClientNotificationCodec notification() throws RPCException {
    try {
      String json = objectMapper.writeValueAsString(this.response.result);
      return new JSONRPCClientNotificationCodec(objectMapper, json.getBytes());
    } catch (IOException e) {
      throw new RPCException("cannot marsharling", e);
    }
  }
  
  @Override
  public <T> T result(Class<T> resultType) throws RPCException {
    String json = null;

    try {
      json = objectMapper.writeValueAsString(this.response.result);
    } catch (IOException e) {
      throw new RPCException("cannot marsharling", e);
    }

    try {
      JavaType targetType = objectMapper.getTypeFactory().constructType(resultType);
      if (!Primitives.isPrimitive(resultType) && !resultType.equals(String.class)) {
        return objectMapper.readValue(json, targetType);
      }
      return objectMapper.convertValue(json, targetType);
    } catch (IOException e) {
      throw new RPCException("cannot unmarsharling", e);
    }
  }
  
  @Override
  public RPCError error() {
    return this.response.error;
  }
  
  @Override
  public Object id() {
    return this.response.id;
  }
}