package com.loafle.commons.rpc.protocol.json;

import java.io.IOException;
import java.lang.reflect.Type;

import com.google.gson.internal.Primitives;
import com.loafle.commons.rpc.RPCException;
import com.loafle.commons.rpc.protocol.RPCError;
import com.loafle.commons.rpc.protocol.RPCServerRequestCodec;

import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.JavaType;

/**
 * JSONRPCServerRequestCodec
 */
class JSONRPCServerRequestCodec implements RPCServerRequestCodec {
  private ObjectMapper objectMapper;
  private JSONServerRequest request;

  public JSONRPCServerRequestCodec(ObjectMapper objectMapper, byte[] buff) throws RPCException {
    this.objectMapper = objectMapper;
    try {
      this.request = this.objectMapper.readValue(buff, JSONServerRequest.class);
    } catch (IOException e) {
      throw new RPCException("cannot unmarsharling", e);
    }
  }

  public String method() {
    return this.request.method;
  }

  public Object[] params(Type[] paramTypes) throws RPCException {
    if (null == paramTypes && null == this.request.params) {
      return null;
    }

    if ((null != paramTypes && null == this.request.params) || null == paramTypes && null != this.request.params) {
      throw new RPCException("params is not valied");
    }

    if (paramTypes.length != this.request.params.size()) {
      throw new RPCException(String.format("The size of params[%d] and types[%d] is not same",
          this.request.params.size(), paramTypes.length));
    }

    Object[] result = new Object[paramTypes.length];

    for (int i = 0; i < paramTypes.length; i++) {
      Type paramType = paramTypes[i];
      String param = this.request.params.get(i);

      JavaType targetType = objectMapper.getTypeFactory().constructType(paramType);
      if (!Primitives.isPrimitive(paramType) && !paramType.getTypeName().equals(String.class.getName())) {
        try {
          result[i] = objectMapper.readValue(param, targetType);
        } catch (IOException e) {
          throw new IllegalArgumentException();
        }
      }
      result[i] = objectMapper.convertValue(param, targetType);
    }

    return result;
  }

  public boolean hasResponse() {
    return null != this.request.id;
  }

  public byte[] response(Object reply, RPCException ex) throws RPCException {
    RPCError rpcError = null;
    if (null != ex) {
      rpcError = new RPCError(1, ex.getMessage(), ex.getData());
    }
    JSONServerResponse response = new JSONServerResponse(JSONRPC.version, reply, rpcError);
    response.id = this.request.id;

    try {
      String json = this.objectMapper.writeValueAsString(response);
      return json.getBytes();
    } catch (IOException e) {
      throw new RPCException("cannot marsharling", e);
    }
  }
}